@@ -98,6 +98,10 @@ public:
     bool isDisabled() const;
     bool isInvalid() const;
 
+    /*! Shorthand for isNull || isRevoked || isExpired ||
+     *                          isDisabled || isInvalid */
+    bool isBad() const;
+
     bool canEncrypt() const;
     /*!
       This function contains a workaround for old gpgme's: all secret
@@ -159,7 +163,7 @@ public:
      * Needs gnupg 2.1.13 and the key needs to be updated
      * afterwards to see the new uid.
      *
-     * @param uid should be fully formated and UTF-8 encoded.
+     * @param uid should be fully formatted and UTF-8 encoded.
      *
      * @returns a possible error.
      **/
@@ -178,6 +182,27 @@ public:
      */
     static Key locate(const char *mbox);
 
+    /* @enum Origin
+     * @brief The Origin of the key. */
+    enum Origin : unsigned int {
+        OriginUnknown = 0,
+        OriginKS = 1,
+        OriginDane = 3,
+        OriginWKD = 4,
+        OriginURL = 5,
+        OriginFile = 6,
+        OriginSelf = 7,
+        OriginOther = 31,
+    };
+    /*! Get the origin of the key.
+     *
+     * @returns the Origin. */
+    Origin origin() const;
+
+    /*! Get the last update time.
+     *
+     * @returns the last update time. */
+    time_t lastUpdate() const;
 private:
     gpgme_key_t impl() const
     {
@@ -229,6 +254,10 @@ public:
     bool isInvalid() const;
     bool isDisabled() const;
 
+    /*! Shorthand for isNull || isRevoked || isExpired ||
+     *                          isDisabled || isInvalid */
+    bool isBad() const;
+
     bool canEncrypt() const;
     bool canSign() const;
     bool canCertify() const;
@@ -334,6 +363,7 @@ public:
     const char *name() const;
     const char *email() const;
     const char *comment() const;
+    const char *uidhash() const;
 
     enum Validity { Unknown = 0, Undefined = 1, Never = 2,
                     Marginal = 3, Full = 4, Ultimate = 5
@@ -345,6 +375,9 @@ public:
     bool isRevoked() const;
     bool isInvalid() const;
 
+    /*! Shorthand for isNull || isRevoked || isInvalid */
+    bool isBad() const;
+
     /** TOFU info for this userid.
      * @returns The TOFU stats or a null TofuInfo.
      */
@@ -371,6 +404,37 @@ public:
      *
      * @returns an error on error.*/
     Error revoke();
+
+    /*! Get the origin of the key.
+     *
+     * @returns the Origin. */
+    Key::Origin origin() const;
+
+    /*! Get the last update time.
+     *
+     * @returns the last update time. */
+    time_t lastUpdate() const;
+
+    /*! Get a remark made by the key provided.
+     * A remark is a signature notation on
+     * this user id made by the key with the
+     * name "rem@gnupg.org". Returns an error if the
+     * parent key of this user id was not listed with the
+     * keylist mode flags for signatures and signature notations.
+     *
+     * @param key The key for which comments should be searched.
+     * @param error Set to GPG_ERR_NO_DATA if the keylist did
+     *              not include signature notations.
+     *
+     * @returns The value of the comment or NULL if none exists.
+     **/
+    const char *remark(const Key &key,
+                       Error &error) const;
+
+    /*! Get multiple remarks made by potentially multiple keys. */
+    std::vector <std::string> remarks(std::vector<GpgME::Key> remarkers,
+                                      Error &error) const;
+
 private:
     shared_gpgme_key_t key;
     gpgme_user_id_t uid;
@@ -403,6 +467,11 @@ public:
         swap(this->sig, other.sig);
     }
 
+    /*! Defines a canonical sort order for signatures of the same user ID. */
+    bool operator<(const Signature &other) const;
+
+    GPGMEPP_DEPRECATED bool operator<(const Signature &other);
+
     bool isNull() const
     {
         return !sig || !uid || !key ;
@@ -423,6 +492,9 @@ public:
     bool isExpired() const;
     bool isExportable() const;
 
+    /*! Shorthand for isNull || isExpired || isInvalid */
+    bool isBad() const;
+
     const char *signerUserID() const;
     const char *signerName() const;
     const char *signerEmail() const;
@@ -449,6 +521,7 @@ private:
 };
 
 GPGMEPP_EXPORT std::ostream &operator<<(std::ostream &os, const UserID &uid);
+GPGMEPP_EXPORT std::ostream &operator<<(std::ostream &os, const Subkey &subkey);
 GPGMEPP_EXPORT std::ostream &operator<<(std::ostream &os, const Key &key);
 
 } // namespace GpgME
