@@ -3,7 +3,8 @@
 
     This file is part of qgpgme, the Qt API binding for gpgme
     Copyright (c) 2008 Klarälvdalens Datakonsult AB
-    Copyright (c) 2016 Intevation GmbH
+    Copyright (c) 2016 by Bundesamt für Sicherheit in der Informationstechnik
+    Software engineering by Intevation GmbH
 
     QGpgME is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -31,6 +32,10 @@
     your version.
 */
 
+#ifdef HAVE_CONFIG_H
+ #include "config.h"
+#endif
+
 #include "qgpgmesignkeyjob.h"
 
 #include "dataprovider.h"
@@ -52,14 +57,17 @@ QGpgMESignKeyJob::QGpgMESignKeyJob(Context *context)
       m_checkLevel(0),
       m_exportable(false),
       m_nonRevocable(false),
-      m_started(false)
+      m_started(false),
+      m_dupeOk(false)
 {
     lateInitialization();
 }
 
 QGpgMESignKeyJob::~QGpgMESignKeyJob() {}
 
-static QGpgMESignKeyJob::result_type sign_key(Context *ctx, const Key &key, const std::vector<unsigned int> &uids, unsigned int checkLevel, const Key &signer, unsigned int opts)
+static QGpgMESignKeyJob::result_type sign_key(Context *ctx, const Key &key, const std::vector<unsigned int> &uids,
+                                              unsigned int checkLevel, const Key &signer, unsigned int opts,
+                                              bool dupeOk, const QString &remark)
 {
     QGpgME::QByteArrayDataProvider dp;
     Data data(&dp);
@@ -68,6 +76,16 @@ static QGpgMESignKeyJob::result_type sign_key(Context *ctx, const Key &key, cons
     skei->setUserIDsToSign(uids);
     skei->setCheckLevel(checkLevel);
     skei->setSigningOptions(opts);
+    skei->setKey(key);
+
+    if (dupeOk) {
+        ctx->setFlag("extended-edit", "1");
+        skei->setDupeOk(true);
+    }
+
+    if (!remark.isEmpty()) {
+        ctx->addSignatureNotation("rem@gnupg.org", remark.toUtf8().constData());
+    }
 
     if (!signer.isNull())
         if (const Error err = ctx->addSigningKey(signer)) {
@@ -88,7 +106,8 @@ Error QGpgMESignKeyJob::start(const Key &key)
     if (m_exportable) {
         opts |= GpgSignKeyEditInteractor::Exportable;
     }
-    run(std::bind(&sign_key, std::placeholders::_1, key, m_userIDsToSign, m_checkLevel, m_signingKey, opts));
+    run(std::bind(&sign_key, std::placeholders::_1, key, m_userIDsToSign, m_checkLevel, m_signingKey, opts,
+                  m_dupeOk, m_remark));
     m_started = true;
     return Error();
 }
@@ -122,4 +141,16 @@ void QGpgMESignKeyJob::setNonRevocable(bool nonRevocable)
     assert(!m_started);
     m_nonRevocable = nonRevocable;
 }
+
+void QGpgMESignKeyJob::setRemark(const QString &remark)
+{
+    assert(!m_started);
+    m_remark = remark;
+}
+
+void QGpgMESignKeyJob::setDupeOk(bool value)
+{
+    assert(!m_started);
+    m_dupeOk = value;
+}
 #include "qgpgmesignkeyjob.moc"
